#!/usr/bin/perl
#############################################################################
#
# WEKO移行用フォーマットへの変換プログラム
# 共通ライブラリ
#
#############################################################################

package common;

use strict;

use Encode;
use lib './';
use Config::Tiny;
use Jcode;

#-----------------------
# 外部変数
#-----------------------
our $log_file_name = "";
our $LOG_INFO = 0;
our $LOG_ERROR = 1;
our $LOG_WARNING = 2;

#############################################################################
# 関数名：param_check
# 設定ファイルからパラメタを取得し、内容のチェックを行い、ハッシュテーブルに展開する。
#############################################################################
sub param_check {
	my $kind = @_[0];
	my $file_name = @_[1];
	my $argments = @_[2];

	# リポジトリ種別のチェック
	if(($kind ne "dspace") && ($kind ne "e-repository") && ($kind ne "nalis")){
		print "[ERROR] リポジトリ種別に誤りがあります。\n";
		print "        リポジトリ種別:dspace\n";
		print "                       e-repository\n";
		print "                       nalis\n";
		return(-1);
	}

	# パラメタファイルの確認
	if(! -r $file_name){
		print "[ERROR] パラメタファイル[$file_name]が見つかりません。\n";
		return(-1);
	}

	# パラメタファイル内のパラメタチェック
	my $init = Config::Tiny->read( $file_name );

	# 共通パラメタ
	$$argments{'start-position'} = $init->{'_'}->{'start-position'};
	$$argments{'output-count'} = $init->{'_'}->{'output-count'};
	$$argments{'max-size'} = uc($init->{'_'}->{'max-size'});
	$$argments{'out-dir'} = $init->{'_'}->{'out-dir'};
	$$argments{'log-file'} = $init->{'_'}->{'log-file'};
	$$argments{'display-count'} = $init->{'_'}->{'display-count'};
	$$argments{'only-metadata'} = $init->{'_'}->{'only-metadata'};
	$$argments{'delimiter'} = $init->{'_'}->{'delimiter'};
	$$argments{'delimiter'} =~ s/^"//;
	$$argments{'delimiter'} =~ s/"*$//;
	$$argments{'compression'} = uc($init->{'_'}->{'compression'});
	$$argments{'difference-mode'} = $init->{'_'}->{'difference-mode'};
	$$argments{'difference-date'} = $init->{'_'}->{'difference-date'};
	$$argments{'type-nii-list'} = $init->{'_'}->{'type-nii-list'};
	# パラメタ値チェック
	if($$argments{'start-position'} =~ /[^0-9]/){
		print "[ERROR] [データ出力位置]の指定に誤りがあります。\n";
		return(-1);
	}
	if($$argments{'output-count'} =~ /[^0-9]/){
		print "[ERROR] [データ出力件数]の指定に誤りがあります。\n";
		return(-1);
	}
	if($$argments{'max-size'} ne ""){
		if($$argments{'max-size'} !~ /^\d+(M|G)$/){
			print "[ERROR] [出力データの容量の上限]の指定に誤りがあります。\n";
			return(-1);
		}
	}
	if($$argments{'difference-mode'} == 1 || $$argments{'difference-mode'} == 2){
		if($$argments{'difference-date'} eq ""){
			print "[ERROR] [差分更新用データ抽出]が指定されていますが「指定日」が指定されていません。\n";
			return(-1);
		}
		if(length($$argments{'difference-date'}) != 8){
			print "[ERROR] (差分抽出)[指定日]に誤りがあります。YYYYMMDDで指定してください。\n";
			return(-1);
		}
	}
	if($$argments{'log-file'} eq ""){
		# 規定値をセット
		$$argments{'log-file'} = "./log/output.log";
	}
	if($$argments{'only-metadata'} =~ /[^0-9]/){
		# 規定値をセット
		$$argments{'only-metadata'} = 0;
	}
	if($$argments{'delimiter'} eq ""){
		# 規定値をセット
		$$argments{'delimiter'} = "|";
	}
	if($$argments{'compression'} eq ""){
		# 規定値をセット
		$$argments{'compression'} = "YES";
	}
	else{
		if(($$argments{'compression'} ne "YES") && ($$argments{'compression'} ne "NO")){
			# 規定値をセット
			$$argments{'compression'} = "YES";
		}
	}

	# DSpaceのパラメタ
	if($kind eq "dspace"){
		$$argments{'db-user'} = $init->{'dspace'}->{'db-user'};
		$$argments{'db-password'} = $init->{'dspace'}->{'db-password'};
		$$argments{'db-name'} = $init->{'dspace'}->{'db-name'};
		$$argments{'db-host'} = $init->{'dspace'}->{'db-host'};
		$$argments{'db-port'} = $init->{'dspace'}->{'db-port'};
		$$argments{'fulltext-dir'} = $init->{'dspace'}->{'fulltext-dir'};
		$$argments{'category-community'} = $init->{'dspace'}->{'category-community'};
		$$argments{'metadata-status'} = $init->{'dspace'}->{'metadata-status'};
		$$argments{'date-field'} = $init->{'dspace'}->{'date-field'};
		$$argments{'author-fieldname'} = $init->{'dspace'}->{'author-fieldname'};
		$$argments{'fulltext-sort'} = $init->{'dspace'}->{'fulltext-sort'};
		$$argments{'niitype-field'} = $init->{'dspace'}->{'niitype-field'};

		# パラメタ値チェック
		if($$argments{'db-user'} eq ""){
			print "[ERROR] [データベース接続ユーザID]が設定されていません。\n";
			return(-1);
		}
		if($$argments{'db-password'} eq ""){
			print "[ERROR] [データベース接続ユーザパスワード]が設定されていません。\n";
			return(-1);
		}
		if($$argments{'db-name'} eq ""){
			print "[ERROR] [データベース名]が設定されていません。\n";
			return(-1);
		}
		if($$argments{'fulltext-dir'} eq ""){
			print "[ERROR] [本文データ格納先ディレクトリ名]が設定されていません。\n";
			return(-1);
		}
		else{
			# ディレクトリの存在チェック
			if(! -d $$argments{'fulltext-dir'}){
				print "[ERROR] 本文データ格納先ディレクトリ[" . $$argments{'fulltext-dir'} . "]は存在しません。\n";
				return(-1);
			}
		}
		if($$argments{'metadata-status'} =~ /[^0-9]/){
			# 規定値をセット
			$$argments{'metadata-status'} = 0;
		}
		if($$argments{'author-fieldname'} eq ""){
			# 規定値をセット
			$$argments{'author-fieldname'} = "dc.contributor.author";
		}
		if($$argments{'fulltext-sort'} =~ /[^0-9]/){
			# 規定値をセット
			$$argments{'fulltext-sort'} = 0;
		}
	}
	elsif ($kind eq "e-repository"){
		$$argments{'db-user'} = $init->{'e-repository'}->{'db-user'};
		$$argments{'db-password'} = $init->{'e-repository'}->{'db-password'};
		$$argments{'db-name'} = $init->{'e-repository'}->{'db-name'};
		$$argments{'fulltext-dir'} = $init->{'e-repository'}->{'fulltext-dir'};
		$$argments{'nii-type'} = $init->{'e-repository'}->{'nii-type'};
		$$argments{'metadata-status'} = $init->{'e-repository'}->{'metadata-status'};
		$$argments{'date-field'} = $init->{'e-repository'}->{'date-field'};

		my @niitypes = ("学術雑誌論文","学位論文","紀要論文","会議発表論文","会議発表用資料","図書","テクニカルレポート","研究報告書","一般雑誌記事","プレプリント","教材","データ・データベース","ソフトウェア","その他");
		my $i=0;
		foreach my $key (@niitypes) {
			my @ja = ();
			for($i=1;;$i++){
				my $str = "level$i";
				my $val = $init->{$key}->{$str};
				if($val){
					push(@ja,$val);
				}
				else{
					last;
				}
			}
			if(scalar(@ja)){
				$$argments{$key} = \@ja;
			}
		}


		# パラメタ値チェック
		if($$argments{'db-user'} eq ""){
			print "[ERROR] [データベース接続ユーザID]が設定されていません。\n";
			return(-1);
		}
		if($$argments{'db-password'} eq ""){
			print "[ERROR] [データベース接続ユーザパスワード]が設定されていません。\n";
			return(-1);
		}
		if($$argments{'fulltext-dir'} eq ""){
			print "[ERROR] [本文データ格納先ディレクトリ名]が設定されていません。\n";
			return(-1);
		}
		else{
			# ディレクトリの存在チェック
			if(! -d $$argments{'fulltext-dir'}){
				print "[ERROR] 本文データ格納先ディレクトリ[" . $$argments{'fulltext-dir'} . "]は存在しません。\n";
				return(-1);
			}
		}
		if($$argments{'metadata-status'} =~ /[^0-9]/){
			# 規定値をセット
			$$argments{'metadata-status'} = 0;
		}
	}
	elsif ($kind eq "nalis"){
		$$argments{'db-user'} = $init->{'nalis'}->{'db-user'};
		$$argments{'db-password'} = $init->{'nalis'}->{'db-password'};
		$$argments{'db-name'} = $init->{'nalis'}->{'db-name'};
		$$argments{'fulltext-dir'} = $init->{'nalis'}->{'fulltext-dir'};
		$$argments{'category-community'} = $init->{'nalis'}->{'category-community'};
		$$argments{'metadata-status'} = $init->{'nalis'}->{'metadata-status'};
		$$argments{'date-field'} = $init->{'nalis'}->{'date-field'};
		$$argments{'author-fieldname'} = $init->{'nalis'}->{'author-fieldname'};
		$$argments{'fulltext-sort'} = $init->{'nalis'}->{'fulltext-sort'};
		$$argments{'niitype-field'} = $init->{'nalis'}->{'niitype-field'};
		# パラメタ値チェック
		if($$argments{'db-user'} eq ""){
			print "[ERROR] [データベース接続ユーザID]が設定されていません。\n";
			return(-1);
		}
		if($$argments{'db-password'} eq ""){
			print "[ERROR] [データベース接続ユーザパスワード]が設定されていません。\n";
			return(-1);
		}
		if($$argments{'fulltext-dir'} eq ""){
			print "[ERROR] [本文データ格納先ディレクトリ名]が設定されていません。\n";
			return(-1);
		}
		else{
			# ディレクトリの存在チェック
			if(! -d $$argments{'fulltext-dir'}){
				print "[ERROR] 本文データ格納先ディレクトリ[" . $$argments{'fulltext-dir'} . "]は存在しません。\n";
				return(-1);
			}
		}
		if($$argments{'metadata-status'} =~ /[^0-9]/){
			# 規定値をセット
			$$argments{'metadata-status'} = 0;
		}
		if($$argments{'author-fieldname'} eq ""){
			# 規定値をセット
			$$argments{'author-fieldname'} = "contributor.author";
		}
		if($$argments{'fulltext-sort'} =~ /[^0-9]/){
			# 規定値をセット
			$$argments{'fulltext-sort'} = 0;
		}
	}
	else{
		print "[ERROR] リポジトリ種別に誤りがあります。\n";
		print "        リポジトリ種別:dspace\n";
		print "                       e-repository\n";
		print "                       nalis\n";
		return(-1);
	}

	return(0);
}

#############################################################################
# 関数名：prohibited_change
# Windowsで使用できない文字を全角に変換する。
#############################################################################
sub prohibited_change {

	my $data = @_[0];

    $data =~ s/[\\]/￥/g;
    $data =~ s/[\/]/／/g;
    $data =~ s/[:]/：/g;
    $data =~ s/[\*]/＊/g;
    $data =~ s/[\?]/？/g;
    $data =~ s/[<]/＜/g;
    $data =~ s/[>]/＞/g;
    $data =~ s/[\|]/｜/g;

	return $data;

}

#############################################################################
# 関数名：normalization
# 文字列の正規化を行う。
#############################################################################
sub normalization {

	my $data = @_[0];

	# データ中の「\n」->「\\n」
	$data =~ s/\\n/\\\\n/g;
	# タブを半角スペースに変換
	$data =~ s/\t/ /g;
	# 改行コードを\nに変換
	$data =~ s/\r\n/\\n/g;
	$data =~ s/\n/\\n/g;
	# 制御コードを削除
	$data =~ s/[\x00-\x1F\x7F]//g;

	return $data;

}

#############################################################################
# 関数名：size_check
# 出力データの容量の上限チェック
#############################################################################
sub size_check {

	my $outdir = @_[0];
	my $max_size = @_[1];

	my $max;

	$max_size = uc($max_size);

	if(substr($max_size, -1) eq "M"){
		# 単位はメガ
		chop($max_size);
		$max = $max_size + 0;
		# キロバイトにする
		$max = $max * 1024;
	}
	elsif (substr($max_size, -1) eq "G"){
		# 単位はギガ
		chop($max_size);
		$max = $max_size + 0;
		# キロバイトにする
		$max = $max * 1024 * 1024;
	}
	else{
		# ここには来ないはず
		$max = 9999999999;
	}

	# 単位をキロバイトに合わせてチェック
	my $a=`du -sk $outdir`;
	my ($use_size,$n)=split(/\t/,$a);
	$use_size = $use_size + 0;

	# 使用量のチェック
	if($max < $use_size){
		# 使用量が上限を超えた
		return(-1);
	}

	return(0);

}


#############################################################################
# 関数名：log_open
# ログファイルの作成を行う。
#############################################################################
sub log_open {

	my $log_file = @_[0];
	$log_file_name = $log_file;

	open(LOG, "> $log_file") or die "[ERROR]ログファイル[$log_file]が作成できません。権限等を見直し再度実行してください。\n$!\n";

	close(LOG);

	return(0);

}

#############################################################################
# 関数名：log_write
# ログファイルの出力を行う。
#############################################################################
sub log_write {

	my $err_kind = @_[0] + 0;
	my $err_msg = @_[1];

	my $kind = "";
	my $out_msg = "";

	# 現在時刻
	my ($sec,$min,$hour,$day,$mon,$year,$wday) = localtime(time);
	$mon++;
	$year += 1900;

	# LOGの種別
	if($err_kind == $LOG_INFO){
		$kind = "INFO";
	}
	elsif($err_kind == $LOG_ERROR){
		$kind = "ERROR";
	}
	elsif($err_kind == $LOG_WARNING){
		$kind = "WARNING";
	}

	if($err_msg){
		$out_msg = sprintf("%04d/%02d/%02d %02d:%02d:%02d ", $year,$mon,$day,$hour,$min,$sec) . "[$kind] $err_msg\n";
		open(LOG, ">>".$log_file_name);
#		binmode LOG,  ":utf8";
		# ファイルに出力
#		print LOG $out_msg;
		print LOG Encode::is_utf8($out_msg)? utf8::encode($out_msg) : $out_msg;
		close(LOG);
		# コンソールにも出力
		print $out_msg;
	}
}

#############################################################################
# 関数名：trim
# 前後の空白を削除
#############################################################################
sub trim {

	my $val = shift;
	$val =~ s/^ *(.*?) *$/$1/;
	return $val;
}

#############################################################################
# 関数名：date_normalize
# 日付フィールドの値をYYYY-MM-DDに変換する
#############################################################################
sub date_normalize {
	my $date_val = shift;
	my $dbuf = "";
	my $year = "";
	my $mon = "";
	my $day = "";

	if($date_val =~ /^[0-9]+$/){
    	if(length($date_val) <= 4){
        	($year,$mon) = ($date_val =~ /(\d{1,4})(.*)/);
        	$dbuf = $year."-01-01";
    	}
    	elsif(length($date_val) <= 6){
        	($year,$mon) = ($date_val =~ /(\d{1,4})(.*)/);
        	$dbuf = $year."-".$mon."-01";
    	}
    	else{
        	($year,$mon,$day) = ($date_val =~ /(\d{1,4})(\d{2})(\d{1,2})(.*)/);
        	$dbuf = $year."-".$mon."-".$day;
    	}
	}
	else{
    	if($date_val =~ /(\d{4})(\D)(\d{1,2})(\D)(\d{1,2})(.*)/){
        	$year = $1;
        	$mon = $3;
        	$day = $5;
        	$dbuf = $year . "-" . $mon . "-" . $day;
    	}
    	elsif($date_val =~ /(\d{4})(\D)(\d{1,2})(.*)/){
        	$year = $1;
        	$mon = $3;
        	$dbuf = $year . "-" . $mon . "-01";
    	}
    	else{
        	($year,$mon) = ($date_val =~ /(\d{1,4})(.*)/);
        	$dbuf = $year."-01-01";
    	}
	}
	return $dbuf;
}

#############################################################################
# 関数名：readfile
# NII資源タイプのリストファイルを読み込み変数に展開する
#############################################################################
sub readfile {
	my $fname = @_[0];		# NII資源タイプのリストファイル名
	my $type_nii_list = @_[1];

	my %value_list = ();

	if(-f $fname){
		if (!open(DATAFILE, $fname)){
			# ファイルオープンエラー
			my $msg = "NII資源タイプファイルがオープンできませんでした。WEKOアイテムタイプの出力はされません。";
			&common::log_write($common::LOG_WARNING,$msg);
			return(-1);
		}
		# 1行ずつ読み込みタブで区切る
		while (my $line = <DATAFILE>){
			chomp($line);
			if($line){
				my($nii,$weko) = split(/\t/,$line);
				$$type_nii_list{$nii} = $weko;
			}
		}
		close(DATAFILE);
	}
	else{
		# NII資源タイプのリストファイルが存在しない -> エラーを出して続行
		my $msg = "NII資源タイプファイルが見つかりませんでした。WEKOアイテムタイプの出力はされません。";
		&common::log_write($common::LOG_WARNING,$msg);
	}

	return(0);

}

#############################################################################
# 関数名：convert
# UTF-8に文字コード変換を行う
#############################################################################
sub convert {

	my $conv_str = @_[0];
	my $ret_str = "";

	$ret_str = &Jcode::convert($conv_str,'utf8',&Jcode::getcode($conv_str));

	return $conv_str;

}

1;

