#############################################################################
#
# WEKO移行用フォーマットへの変換プログラム
# DSpace用モジュール
#
#############################################################################

package RepositoryControl;

use strict;

use DBI;

use lib qw(./);
use common;

my ($context);
my ($items_sth);
my ($sth_item_id,$sth_item_handle,$sth_in_archive,$sth_withdrawn,$sth_last_modified,$sth_collection_handle,$sth_collection_id);
my ($g_col_com_id) = 0;
my ($dspace_ver) = "0";
my (@tag_table) = ();
my (@date_field) = ();
my (%prohibited_change_flg) = ();	# Windows使用不可文字ワーニング出力済みフラグ
#====================================================================
# 関数名：db_connect
# 機  能：データベースに接続する
#====================================================================
sub db_connect {
	my $argments = @_[0];

	my $dbuser = $$argments{'db-user'};
	my $dbpassword = $$argments{'db-password'};
	my $dbname = $$argments{'db-name'};
	my $dbhost = $$argments{'db-host'};
	my $dbport = $$argments{'db-port'};
	my $msg = "";

	# DBホスト未設定の場合は「localhost」セット
	if($dbhost eq ""){
		$dbhost = "localhost";
	}
	# DBポート未設定の場合は「5432」セット
	if($dbport eq ""){
		$dbport = "5432";
	}

	my $psql_connecter = "DBI:Pg:dbname=$dbname;host=$dbhost;port=$dbport";
	$context = DBI->connect($psql_connecter, $dbuser, $dbpassword, {RaiseError => 1, PrintError => 0});
	if ( ! $context ){
		my $msg = "データベースの接続に失敗しました。[$context->err $context->errstr]";
		&common::log_write($common::LOG_ERROR,$msg);
		return(-1);
	}

	return(0);
}



#====================================================================
# 関数名：db_final
# 機  能：データベースから切断する
#====================================================================
sub db_final {
	if($items_sth){
		$items_sth->finish();
	}
	$context->disconnect;
}


#====================================================================
# 関数名：db_count
# 機  能：該当件数を返す
#====================================================================
sub db_count {
	my $argments = @_[0];
	my $msg = "";


	# DSpaceのバージョンチェック
	# 返戻 1:バージョン1.6以上 0:バージョン1.6未満
	$dspace_ver = &check_version();

	my $sql1 = "SELECT * FROM" .
        "(" .
        "  SELECT" .
        "    distinct item_id," .
        "    in_archive," .
        "    withdrawn," .
        "    last_modified," .
        "    handle AS collection_handle," .
        "    owning_collection" .
        "  FROM" .
        "    (" .
        "     SELECT" .
        "       distinct item_id," .
        "       handle AS item_handle," .
        "       in_archive," .
        "       withdrawn," .
        "       last_modified," .
        "       owning_collection" .
        "     FROM" .
        "       item" .
        "     LEFT OUTER JOIN" .
        "       handle" .
        "     ON item.item_id = handle.resource_id AND" .
        "        handle.resource_type_id = 2" .
        "    ) A" .
        "  LEFT OUTER JOIN" .
        "    handle" .
        "  ON A.owning_collection = handle.resource_id AND" .
        "     handle.resource_type_id = 3";

	# 出力対象のステータス確認
	my $sql2 = "";
	if($$argments{'metadata-status'} == 1){
		# 削除済みのみ出力
		$sql2 = ") B" .
        	" WHERE" .
        	"  (in_archive = false AND withdrawn = true) ";
	}
	elsif($$argments{'metadata-status'} == 2){
		# 編集中データのみ
		$sql2 = ") B" .
        	" WHERE" .
        	"  (in_archive = false AND withdrawn = false) ";
	}
	elsif($$argments{'metadata-status'} == 3){
		# 全て出力
		$sql2 = ") B" .
        	" WHERE" .
        	"   ((in_archive = true AND withdrawn = false) or (in_archive = false AND withdrawn = true) or (in_archive = true AND withdrawn = true) or (in_archive = false AND withdrawn = false)) "; 
	}
	else{
		# 公開済みデータのみ
		$sql2 = ") B" .
        	" WHERE" .
        	"  (in_archive = true AND withdrawn = false) ";
	}

	# 差分更新抽出あり?
	my $difference_sql = "";
	if($$argments{'difference-mode'} == 1){	# 新規登録データのみ
		my($year,$mon,$day) = ($$argments{'difference-date'} =~ /(\d{4})(\d{2})(\d{2})/);
		my $difference_date = $year . "-" . $mon . "-" . $day;
		$difference_sql = " AND item_id in (select distinct item_id from metadatavalue where metadata_field_id in (select metadata_field_id from metadatafieldregistry where element='date' and qualifier='accessioned') AND substr(text_value,1,10) >= '".$difference_date."') ";
	}
	elsif($$argments{'difference-mode'} == 2){	# 更新データのみ
		my($year,$mon,$day) = ($$argments{'difference-date'} =~ /(\d{4})(\d{2})(\d{2})/);
		my $difference_date = $year . "-" . $mon . "-" . $day;
		$difference_sql = " AND (to_char(last_modified,'YYYY-MM-DD') >= '".$difference_date."') AND item_id in (select distinct item_id from metadatavalue where metadata_field_id in (select metadata_field_id from metadatafieldregistry where element='date' and qualifier='accessioned') AND substr(text_value,1,10) < '".$difference_date."') ";
	}

	# ソート
	my $sort_sql = " ORDER BY item_id, in_archive, withdrawn";


	my $comm_coll_sql = "";
	$g_col_com_id = 0;

	# コミュニティまたはコレクション指定の場合
	my $comm_handle = $$argments{'category-community'};
	if($comm_handle ne ""){
		# ハンドル番号からリソース番号を取得
		my @bindVars = ();
		push(@bindVars, $comm_handle);
		my $com_sql = "select resource_id,resource_type_id from handle where handle_id = ?";
		my @result = &db_select($com_sql, @bindVars);
		if(scalar(@result) <= 0){
			my $msg = "指定されたコミュニティまたはコレクション[$comm_handle]は存在しません。";
			&common::log_write($common::LOG_ERROR,$msg);
			return(-1);
		}

		my ($resource_id,$resource_type_id) = split(/\t/,$result[0]);
		$g_col_com_id = $resource_id;
		if($resource_type_id == 4){
			# 指定されたのはコミュニティ
			$comm_coll_sql = " WHERE item_id in (SELECT distinct item_id FROM communities2item WHERE community_id = $resource_id) ";
		}
		elsif ($resource_type_id == 3){
			# 指定されたのはコレクション
			$comm_coll_sql = " WHERE item_id in (SELECT distinct item_id FROM collection2item WHERE collection_id = $resource_id) ";
		}
	}

	# 作成したSQLを結合
	my $sql = "$sql1$comm_coll_sql$sql2$difference_sql$sort_sql"; 

	# データ出力件数が指定されている場合
	my $limit = $$argments{'output-count'} + 0;
	if($limit > 0){
		$sql .= " limit $limit";
	}

	# データ出力位置が指定されている場合
	my $offset = $$argments{'start-position'} + 0;
	if($offset > 0){
		$offset = $offset - 1;
		$sql .= " offset $offset";
	}

	$items_sth = $context->prepare($sql);
	eval {
		$items_sth->execute or die;
	};
	if($@){
		my $estr = $DBI::errstr;
		chomp($estr);
		my $msg = "データベースでエラーが発生しました。[$estr]";
		&common::log_write($common::LOG_ERROR,$msg);
		return(-1);
	}

	$items_sth->bind_col( 1, \$sth_item_id);
	$items_sth->bind_col( 2, \$sth_in_archive);
	$items_sth->bind_col( 3, \$sth_withdrawn);
	$items_sth->bind_col( 4, \$sth_last_modified);
	$items_sth->bind_col( 5, \$sth_collection_handle);
	$items_sth->bind_col( 6, \$sth_collection_id);

	my $num = $items_sth->rows;

	# 日付のフォーマット変換の指定あり？
	@date_field = split(/,/,$$argments{'date-field'});
	for(my $i=0; $i<scalar(@date_field)-1; $i++){
		$date_field[$i] = &common::trim($date_field[$i]);
	}

	return($num);
}


#====================================================================
# 関数名：db_select
# 機  能：指定された SQL 文を実行し、結果を TSV の配列で返却する
#====================================================================
sub db_select {
    my ($sql, @var) = @_;
    my (@iterator) = ();
    my (@buffer) = ();
    my ($stmt, $fields, $lp);

    $stmt = $context->prepare($sql);

    if ( $#var != -1 ) {
        $lp = 1;
        foreach (@var) {
            $stmt->bind_param( $lp, $_ );
            $lp++;
        }
    }

	eval {
    	$stmt->execute or die;
	};
	if($@){
		my $estr = $DBI::errstr;
		chomp($estr);
		my $msg = "データベースでエラーが発生しました。[$estr]";
		&common::log_write($common::LOG_ERROR,$msg);
		return(-1);
	}

    while ( @buffer = $stmt->fetchrow_array ) {
        $fields = "";
        $lp = 0;
        foreach (@buffer) {
            $fields .= "\t" unless ($lp == 0);
            $fields .= &sanitaryColumns($_);
            $lp++;
        }
        push(@iterator, $fields);
    }
    $stmt->finish ;
    return @iterator;
}

#====================================================================
# 関数名：sanitaryColumns
# 機  能：UTF-8エンコーディングを行う
#====================================================================
sub sanitaryColumns {
    my (@colmuns) = @_ ;
    my (@results) = ();
    my ($buff) = "";

    for (@colmuns) {
        utf8::encode($_) if ( utf8::is_utf8($_) );
    }
    return wantarray ? @colmuns : $colmuns[0];

}

#====================================================================
# 関数名：db_getTags
# 機  能：タグデータをTAB区切りで全出力
#====================================================================
sub db_getTags {

	my ($metadata_field_id,$element,$qualifier,$short_id);
	my $line = "";
	my $tags = "";

	my $tag_sql = "select metadatafieldregistry.metadata_field_id,metadatafieldregistry.element,metadatafieldregistry.qualifier,metadataschemaregistry.short_id from metadatafieldregistry,metadataschemaregistry where metadatafieldregistry.metadata_schema_id = metadataschemaregistry.metadata_schema_id order by metadata_field_id";
	my @results = &db_select($tag_sql);

	my $i = 0;
	foreach my $result (@results) {
		$metadata_field_id = "";
		$element = "";
		$qualifier = "";

		($metadata_field_id,$element,$qualifier,$short_id) = split(/\t/,$result);
		$tags = "$short_id.$element";
		if($qualifier){
			$tags .= ".$qualifier";
		}
		if($line){
			$line .= "\t";
		}
		$line .= $tags;

		$tag_table[$i] = $metadata_field_id;
		$i++;
	}

	# 日付のフォーマット変換の指定あり？
	my $df = "";
	for(my $i=0; $i<scalar(@date_field); $i++){
		if($date_field[$i]){
			$df .= "\t".&common::trim($date_field[$i]).".normalize";
		}
	}
	if($df){
		$line .= $df;
	}

	# DSpace1.6以上の場合は著者ID出力
	if($dspace_ver eq "1"){
		$line .= "\t著者ID";
	}

	# WEKO用タグ：公開日とFEEDBACK_MAILとPOS_INDEXとITEM_KEYとファイルを付加
	$line .= "\t公開日\t更新日\tFEEDBACK_MAIL\tPOS_INDEX\tITEM_KEY\tファイル\tファイル（説明）\tWEKOアイテムタイプ";

	return $line;
}

#====================================================================
# 関数名：db_getData
# 機  能：1メタデータをTAB区切りで全出力
#====================================================================
sub db_getData {

	my $argments = @_[0];
	my $type_nii_list = @_[1];
	my $item_sth;

	$items_sth->fetchrow_arrayref;

	my $elem="";
	my $qual="";
	my $txtv="";
	my $mfld="";
	my $shortid="";
	my $wkidx=0;
	my $dtcnt=0;
	my @wkdata=();
	my $line="";
	my $pre_metaf_id="";
	my $msg = "";
	my $com_col_flg=0;
	my @comm_array = ();
	my $date_available = "";
	my $tag_name = "";
	my $last_modified = "";
	my $authority_id = "";
	my $weko_item = "";


	# コミュニティまたはコレクション指定値を取得
	if($$argments{'category-community'}){
		$com_col_flg = 1;
	}

	# データ取得
	my $sql = "SELECT a.element," .
            " CASE" .
                " WHEN a.qualifier IS NOT NULL THEN a.qualifier" .
                " ELSE 'none'" .
                " END," .
            " b.text_value," .
            " a.metadata_field_id," .
            " c.short_id," .
            " to_char(d.last_modified,'YYYY-MM-DD')";

	if($dspace_ver eq "1"){
		$sql .= ", b.authority";
	}


	$sql .= " FROM metadatafieldregistry a, metadatavalue b, metadataschemaregistry c, " .
			" item d ".
            " WHERE a.metadata_field_id = b.metadata_field_id" .
			" AND a.metadata_schema_id = c.metadata_schema_id" .
            " AND b.item_id=$sth_item_id AND d.item_id=$sth_item_id" .
			" ORDER BY a.metadata_field_id, b.place";

	$item_sth = $context->prepare($sql);
	eval {
		$item_sth->execute or die;
	};
	if($@){
		my $estr = $DBI::errstr;
		chomp($estr);
		my $msg = "データベースでエラーが発生しました。[$estr]";
		&common::log_write($common::LOG_ERROR,$msg);
		exit;
	}
	$item_sth->bind_col(1, \$elem);
	$item_sth->bind_col(2, \$qual);
	$item_sth->bind_col(3, \$txtv);
	$item_sth->bind_col(4, \$mfld);
	$item_sth->bind_col(5, \$shortid);
	$item_sth->bind_col(6, \$last_modified);
	if($dspace_ver eq "1"){
		$item_sth->bind_col(7, \$authority_id);
	}

	$date_available = "";
	my @date_field_value = ();
	my @authority = ();

	# 繰り返し項目はデリミタ(パラメタファイルで指定)で接続する
   	while ( $item_sth->fetchrow_arrayref ) {

		# 日付のフォーマット変換あり
		$tag_name = "$shortid.$elem";
		if($qual){
			$tag_name .= ".$qual";
		}
		for(my $i=0; $i<scalar(@date_field); $i++){
			# 指定のタグ名と同一か?
			if($date_field[$i] eq $tag_name){
				if($txtv){
					my $buf = "";
					# 日付正規化関数呼び出し
					$buf = &common::date_normalize($txtv);
					if($buf){
						if($date_field_value[$i]){
							$date_field_value[$i] .= $$argments{'delimiter'};
						}
						$date_field_value[$i] .= $buf;
					}
				}
				last;
			}
		}

		# date.availableを「公開日」とする
		if( ($elem eq "date") && ($qual eq "available") && $txtv){
			my($year,$mon,$day, $hour) =
        		($txtv =~ /(\d{4})\-(\d\d)\-(\d\d)(.*)/);
			$date_available = $year . "-" . $mon . "-" . $day;
		}

		if($dspace_ver eq "1"){
			if($tag_name eq $$argments{'author-fieldname'}){
				push(@authority,$authority_id);
			}
		}

		# NII資源タイプからWEKOアイテムタイプ出力
		my $niitype = $$argments{'niitype-field'};	# パラメタファイルのNIIタグ名
		my $metadataf = "$shortid.$elem.$qual";		# メタデータタグ名
		if($niitype){
			if( $metadataf eq $niitype ){
				# 指定されたNII資源タイプのメタデータタグ名と同じ場合
				$weko_item = $$type_nii_list{$txtv};
			}
		}

		# データの正規化
		#  - タブを半角スペースに変換
		#  - 改行コードを\nに変換
		#  - 制御コードを削除
		#  - データ中の「\n」->「\\n」
		$txtv = &common::normalization($txtv);
       	if ($pre_metaf_id eq $mfld) {
           	$wkdata[$wkidx] .= $$argments{'delimiter'}."$txtv";
       	} else {
           	for (my $x = 0; $x <= scalar(@tag_table)-1; $x++) {
               	if ($tag_table[$x] eq $mfld) {
                   	$wkdata[$x] = $txtv;
                   	$pre_metaf_id = $mfld;
                   	$wkidx = $x;
                   	$dtcnt++;
                   	last;
               	}
           	}
       	}
   	}

	$item_sth->finish();

	if ($dtcnt > 0) {
		for (my $x = 0; $x <= scalar(@tag_table)-1; $x++) {
            $line .= $wkdata[$x];
			$line .= "\t";
		}

		# 日付のフォーマット変換あり
		for(my $i=0; $i<scalar(@date_field); $i++){
			$line .= $date_field_value[$i];
			$line .= "\t";
		}

		# 著者ID
		if($dspace_ver eq "1"){
			$line .= join($$argments{'delimiter'},@authority);
			$line .= "\t";
		}

		# 公開日
		$line .= $date_available;
		$line .= "\t";

		# 最終更新日
		$line .= $last_modified;
		$line .= "\t";

		# FEEDBACK_MAIL(統計提供用ユーザのメールアドレス)の出力
		my $email = "";
		my $e_person = "";
		$sql = "select email from eperson where eperson_id in (select distinct supplier_id from item2supplier where item_id =  $sth_item_id) order by eperson_id";
		$item_sth = $context->prepare($sql);
		eval {
			$item_sth->execute or die;
		};
		if($@){
			# 機関によってはitem2supplierを持っていない場合がある
			# その場合はFEEDBACK_MAILは空で出力
#			my $msg = $DBI::errstr;
			my $errcode = $DBI::err;
#			if($msg =~ /relation "item2supplier" does not exist/){
			if($errcode == 7){
				$e_person = "";
			}
			else{
				my $estr = $DBI::errstr;
				chomp($estr);
				my $msg = "データベースでエラーが発生しました。[$estr]";
				&common::log_write($common::LOG_ERROR,$msg);
				exit;
			}
		}
		else{
			$item_sth->bind_col(1, \$email);
			while ( $item_sth->fetchrow_arrayref ) {
				if($e_person ne ""){
					$e_person .= $$argments{'delimiter'};
				}
				$e_person .= $email;
			}
      		$item_sth->finish();
		}
		$line = "$line$e_person";

		# コミュニティ/コレクションの出力
		my $temp_collection_id = "";
		my @coll_ids = ();
		@comm_array = ();

		$sql = "select collection_id from collection2item where item_id =  $sth_item_id order by id";
		$item_sth = $context->prepare($sql);
		eval {
			$item_sth->execute or die;
		};
		if($@){
			my $estr = $DBI::errstr;
			chomp($estr);
			my $msg = "データベースでエラーが発生しました。[$estr]";
			&common::log_write($common::LOG_ERROR,$msg);
			exit;
		}
		$item_sth->bind_col(1, \$temp_collection_id);
		while ( $item_sth->fetchrow_arrayref ) {
			push(@coll_ids,$temp_collection_id);
		}
      	$item_sth->finish();


		foreach my $coll_id (@coll_ids) {

			my $com_col_out_flg = 0;
       		my @category = ();
			my $collection_name = "";
			my $community_id = "";

			# 特定のコミュニティまたはコレクションが指定されている
			if($com_col_flg == 1){
				# 指定のコミュニティまたはコレクションと同じ
				if($coll_id == $g_col_com_id){
					# 出力対象フラグON
					$com_col_out_flg = 1;
				}
			}

       		# コレクション名を取得
       		$sql = "SELECT name FROM collection WHERE collection_id = ?";
       		$item_sth = $context->prepare($sql);
			eval {
       			$item_sth->execute($coll_id) or die;
			};
			if($@){
				my $estr = $DBI::errstr;
				chomp($estr);
				my $msg = "データベースでエラーが発生しました。[$estr]";
				&common::log_write($common::LOG_ERROR,$msg);
				exit;
			}
       		$item_sth->bind_col(1, \$collection_name);
       		while ( $item_sth->fetchrow_arrayref ) {
				push(@category, "$collection_name");
				last;
       		}
       		$item_sth->finish();

       		# コレクション直のコミュニティIDを取得
       		$item_sth = $context->prepare("SELECT community_id FROM community2collection WHERE collection_id = ?");
			eval {
       			$item_sth->execute($coll_id) or die;
			};
			if($@){
				my $estr = $DBI::errstr;
				chomp($estr);
				my $msg = "データベースでエラーが発生しました。[$estr]";
				&common::log_write($common::LOG_ERROR,$msg);
				exit;
			}
       		$item_sth->bind_col(1, \$community_id);
       		while ( $item_sth->fetchrow_arrayref ) {
           		last;
       		}
       		$item_sth->finish();

       		# サブコミュニティがあればそのハンドル値と名前を取得
       		while ($community_id > 0) {

				# 特定のコミュニティまたはコレクションが指定されている
				if($com_col_flg == 1){
					# 指定のコミュニティまたはコレクションと同じ
					if($community_id == $g_col_com_id){
						# 出力対象フラグON
						$com_col_out_flg = 1;
					}
				}

				my $community_name="";
				my $community_handle="";
           		$sql = "SELECT c.name, h.handle FROM community c, handle h WHERE " .
                   "c.community_id = ? AND h.resource_type_id = 4 AND h.resource_id  = ?";

           		$item_sth = $context->prepare($sql);
				eval {
           			$item_sth->execute($community_id, $community_id) or die;
				};
				if($@){
					my $estr = $DBI::errstr;
					chomp($estr);
					my $msg = "データベースでエラーが発生しました。[$estr]";
					&common::log_write($common::LOG_ERROR,$msg);
					exit;
				}
           		$item_sth->bind_col(1, \$community_name);
           		$item_sth->bind_col(2, \$community_handle);

           		while ( $item_sth->fetchrow_arrayref ) {
               		push(@category, "$community_name");
           		}

           		$item_sth->finish();

           		# 親コミュニティがあるか調査
           		$community_id = &parent_com($community_id);

       		}

			# 全出力か指定されたコミュニティまたはコレクションが含まれる場合のみ出力
			if(($com_col_flg == 0) || (($com_col_flg == 1) && ($com_col_out_flg == 1))){
       			my $comstr = "";
       			foreach my $xx (reverse @category) {
					my $normalize_community = "";
					# Windows禁止文字のチェック
					$normalize_community = &common::prohibited_change($xx);
					if($normalize_community ne $xx){
						# 同じコミュニティ・コレクションが続く場合が多いので一度出力したものは出力しないようにする
						if($prohibited_change_flg{$normalize_community} != 1){
							my $msg = "コミュニティ・コレクション名にWindowsで使用できない文字が含まれていたので変換しました。[$xx] -> [$normalize_community]";
							&common::log_write($common::LOG_WARNING,$msg);
						}
						$prohibited_change_flg{$normalize_community} = 1;
					}
           			if ($comstr eq "") {
               			$comstr = $normalize_community;
           			} else {
               			$comstr .= "/$normalize_community";
           			}
       			}
				push(@comm_array,$comstr);
			}
		}
	}

	return $sth_item_id,$line,$weko_item,\@comm_array;

}

#################################################################
# 親コミュニティ検索処理
#################################################################
sub parent_com {

    my $comid = shift;
    my $parent_id = -1;

    my $sth = $context->prepare("SELECT parent_comm_id FROM community2community WHERE child_comm_id = ?");
	eval {
    	$sth->execute($comid) or die;
	};
	if($@){
		my $estr = $DBI::errstr;
		chomp($estr);
		my $msg = "データベースでエラーが発生しました。[$estr]";
		&common::log_write($common::LOG_ERROR,$msg);
		exit;
	}
    $sth->bind_col(1, \$parent_id);

    while ( $sth->fetchrow_arrayref ) {
        last;
    }

    $sth->finish();

    return $parent_id;

}


#################################################################
# 本文ファイル取得、ディレクトリ作成、コピー
#################################################################
sub db_getFulltext {

    my ($item_key, $topdir, $argments) = @_;

	my $item_sth;
	my $fulltext_path = "";
	my $fulltext_file_path = "";
	my $fulltext_file = "";
	my @file_names = ();
	my $msg = "";
	my $normalize_fname = "";
	my @fname_array = ();
	my @fdescription_array = ();

	$fulltext_path = $$argments{'fulltext-dir'};
	if(!$fulltext_path){
		my $msg = "本文データ格納先ディレクトリ名が設定されていません。";
		&common::log_write($common::LOG_ERROR,$msg);
		return(-1);
	}
	if(! -d $fulltext_path){
		my $msg =  "本文データ格納先ディレクトリ[$fulltext_path]は存在しません。";
		&common::log_write($common::LOG_ERROR,$msg);
		return(-1);
	} 

	# 1次ファイルのソート
	my $bitstream_sort = "";
	if($$argments{'fulltext-sort'} == 1){
		$bitstream_sort = "ASC";
	}
	else{
		$bitstream_sort = "DESC";
	}

   	# 1次ファイル取得
   	my $sql = "SELECT" .
                "  bitstream.bitstream_id," .
                "  bitstream.sequence_id," .
                "  bitstream.name," .
                "  bitstream.internal_id," .
                "  bitstream.description" .
                " FROM" .
                "  bitstream," .
                "  bundle2bitstream," .
                "  bundle," .
                "  item2bundle" .
                " WHERE" .
                "  bitstream.bitstream_id = bundle2bitstream.bitstream_id AND" .
                "  bundle2bitstream.bundle_id = bundle.bundle_id AND" .
                "  bundle.bundle_id = item2bundle.bundle_id AND" .
                "  item2bundle.item_id = $item_key AND" .
                "  bundle.name = 'ORIGINAL'" .
                " ORDER BY bitstream.sequence_id $bitstream_sort";

   	my $bitstream_id = "";
    my $name = "";
   	my $internal_id = "";
   	my $description = "";
   	my $seq_id = "";

   	$item_sth = $context->prepare($sql);
	eval {
   		$item_sth->execute or die;
	};
	if($@){
		my $estr = $DBI::errstr;
		chomp($estr);
		my $msg = "データベースでエラーが発生しました。[$estr]";
		&common::log_write($common::LOG_ERROR,$msg);
		exit;
	}
   	$item_sth->bind_col(1, \$bitstream_id);
   	$item_sth->bind_col(2, \$seq_id);
   	$item_sth->bind_col(3, \$name);
   	$item_sth->bind_col(4, \$internal_id);
   	$item_sth->bind_col(5, \$description);

	my $num = $item_sth->rows;

	if($num > 0){

		# 重複ファイル名の取得
		my $file_sth;
		if($num > 1){
			$sql = "SELECT" .
					"  bitstream.name," .
					"  count(bitstream.name) as cnt" .
					" FROM" .
                	"  bitstream," .
                	"  bundle2bitstream," .
                	"  bundle," .
                	"  item2bundle" .
                	" WHERE" .
                	"  bitstream.bitstream_id = bundle2bitstream.bitstream_id AND" .
                	"  bundle2bitstream.bundle_id = bundle.bundle_id AND" .
                	"  bundle.bundle_id = item2bundle.bundle_id AND" .
                	"  item2bundle.item_id = $item_key AND" .
                	"  bundle.name = 'ORIGINAL'" .
					" GROUP BY bitstream.name having(count(bitstream.name) > 1)" .
                	" ORDER BY bitstream.name";

    		my $bname = "";
   			my $bcnt = "";

   			$file_sth = $context->prepare($sql);
			eval {
   				$file_sth->execute or die;
			};
			if($@){
				my $estr = $DBI::errstr;
				chomp($estr);
				my $msg = "データベースでエラーが発生しました。[$estr]";
				&common::log_write($common::LOG_ERROR,$msg);
				exit;
			}
   			$file_sth->bind_col(1, \$bname);
   			$file_sth->bind_col(2, \$bcnt);

			my $i = 0;

   			while ( $file_sth->fetchrow_arrayref ) {
				# 重複しているファイル名と枝番の初期値をセット
				$file_names[$i][0] = $bname;
				$file_names[$i][1] = 1;
				$i++;
			}
			$file_sth->finish();
		}

		# 「メタデータのみを出力する」の場合は実施しない
		if($$argments{'only-metadata'} != 1){

			# 本文ファイル格納用ディレクトリの作成
			$fulltext_file_path = "$topdir/$item_key";
			$! = "";
			mkdir $fulltext_file_path;
			if ($!) {
				# ディレクトリ作成エラー
				my $msg = "[$topdir]配下にディレクトリが作成できません。権限等を見直し再度実行してください。エラーコード[$!] ディレクトリ名[$topdir/$item_key]";
				&common::log_write($common::LOG_ERROR,$msg);
				return(-1);
			}
		}
	

   		# 本文ファイルをコピーする
   		while ( $item_sth->fetchrow_arrayref ) {

			my $flg = 0;

			# 重複ファイルの確認
			for(my $i=0; $i<scalar(@file_names); $i++){
				# 重複ファイルに該当
				if($file_names[$i][0] eq $name){
					my $fname="";
					my $suffix="";
					if ($name=~ /(.*).(.*)/) {
						$fname = $1;
						$suffix = ".$2";
					}
					else{
						$fname = $name;
					}
					# 枝番を付けたファイル名の作成
					my $edaban = $file_names[$i][1];
					$name = "$name" . "_" . sprintf("%02d",$edaban) . $suffix;

					$edaban++;
					$file_names[$i][1] = $edaban;	# 枝番アップ
					$flg = 1;
				}
			}

			# 複数ある場合は | で接続する
			if ($name) {
				# Windows禁止文字のチェック
				$normalize_fname = &common::prohibited_change($name);
				if($normalize_fname ne $name){
					my $msg = "本文ファイル名にWindowsで使用できない文字が含まれていたので変換しました。ITEM_KEY:$item_key [$name] -> [$normalize_fname]";
					&common::log_write($common::LOG_WARNING,$msg);
				}
				push(@fname_array,$normalize_fname);
				push(@fdescription_array,$description);
			}

			# 「メタデータのみを出力する」の場合は実施しない
			if($$argments{'only-metadata'} != 1){

				# 本文ファイルのフルパス作成
				$fulltext_file = $fulltext_path . "/" . substr($internal_id,0,2) . "/" . substr($internal_id,2,2) . "/" . substr($internal_id,4,2) . "/" . $internal_id;

				if(-f $fulltext_file){
					# ファイルコピー
					$? = 0;
					$normalize_fname =~ s/["]/\\"/g;
					my $ret = system("cp $fulltext_file \"$fulltext_file_path/$normalize_fname\"");
					if ($? != 0){
						my $msg = "本文ファイルのコピーに失敗しました。エラーコード:$? ITEM_KEY:$item_key From:$fulltext_file To:$fulltext_file_path/$normalize_fname";
						&common::log_write($common::LOG_ERROR,$msg);
						return(-1);
					}
					my $ret = system("chmod 664 \"$fulltext_file_path/$normalize_fname\"");
				}
				else{
					# 本文データが存在しない -> エラーを出して続行
					my $msg = "本文ファイルが見つかりませんでした。ITEM_KEY:$item_key ファイル名:$name($internal_id)";
					&common::log_write($common::LOG_ERROR,$msg);
				}
			}
		}
	}

	$item_sth->finish();

	return(\@fname_array, \@fdescription_array);

}

#====================================================================
# 関数名：db_getEperson
# 機  能：epersonのデータをTAB区切りで全出力
#====================================================================
sub db_getEperson {

	my $topdir = @_[0];

	my ($firstname,$lastname,$email);
	my ($eprson_sth);

	#------------------------------------------
	# tsvファイル作成(BOM付きで作成)
	#------------------------------------------
	my $out_file = "$topdir/eperson.tsv";
	unless(open(EPERSON, "> $out_file")){
    	my $msg = "[$topdir]配下にファイルが作成できません。権限等を見直し再度実行してください。エラーコード:$!";
    	&common::log_write($common::LOG_ERROR,$msg);
		return(-1);
	}
	my $bom = "\x{feff}"; #BOM
	utf8::encode($bom);
	print EPERSON $bom;
	#  1行目出力
	print EPERSON "firstname\tlastname\temail\r\n";

	my $sql = "SELECT firstname,lastname,email FROM eperson order by eperson_id";
	$eprson_sth = $context->prepare($sql);
	eval {
		$eprson_sth->execute or die;
	};
	if($@){
		my $estr = $DBI::errstr;
		chomp($estr);
		my $msg = "データベースでエラーが発生しました。[$estr]";
		&common::log_write($common::LOG_ERROR,$msg);
		exit;
	}
	$eprson_sth->bind_col(1, \$firstname);
	$eprson_sth->bind_col(2, \$lastname);
	$eprson_sth->bind_col(3, \$email);

	my $i = 0;
	my $line_data;

	# firstname,lastname,emailをタブ区切りで出力
   	while ( $eprson_sth->fetchrow_arrayref ) {
		$line_data = "";

		$line_data = "$firstname\t$lastname\t$email\r\n";

		print EPERSON $line_data;
	}

	close(EPERSON);

	return(0);

}

#====================================================================
# 関数名：check_version
# 機  能：DSpaceのバージョン確認
#         metadatavalueテーブルにauthority列がある場合は1.6以上とする
#====================================================================
sub check_version {

	my $sql = "select count(column_name) from information_schema.columns where table_name = 'metadatavalue' and column_name = 'authority'";

	my $column_count = "";
	my $ary_ref = $context->selectrow_arrayref($sql);
	$column_count = $ary_ref->[0];

	return $column_count;

}

1;

