#############################################################################
#
# WEKO移行用フォーマットへの変換プログラム
# E-Repository用モジュール
#
#############################################################################

package RepositoryControl;

use strict;


use DBI;
use Encode;

use lib qw(./);
use common;

$ENV{'NLS_LANG'}='Japanese_Japan.UTF8';

my ($context);
my ($items_sth);
my ($sth_item_id,$sth_mtcom,$sth_crtid,$sth_rnwid,$sth_mtstat,$sth_mtpermit,$sth_adate,$sth_udate,$sth_utime);
my ($g_col_com_id) = 0;
my (@tag_table) = ();
my %tnii=();
my %tnii_name=();
my %tnii_name_e=();
my %torg=();
my %torg_name=();
my (@date_field) = ();
my (%prohibited_change_flg) = ();   # Windows使用不可文字ワーニング出力済みフラグ
#====================================================================
# 関数名：db_connect
# 機  能：データベースに接続する
#====================================================================
sub db_connect {
	my $argments = @_[0];

	my $dbuser = $$argments{'db-user'};
	my $dbpassword = $$argments{'db-password'};
	my $dbname = $$argments{'db-name'};
	my $msg = "";

	if($dbname eq "-"){
		$dbname = "DBI:Oracle";
	}
	else{
		$dbname = "DBI:Oracle:$dbname";
	}

	$context = DBI->connect($dbname, $dbuser, $dbpassword, {RaiseError => 0, PrintError => 0});
	if ( ! $context ){
		my $msg = "データベースの接続に失敗しました。[".&common::convert($DBI::errstr)."]";
		&common::log_write($common::LOG_ERROR,$msg);
		return(-1);
	}

	$context->{LongReadLen} = 1024 * 100;

	return(0);
}


#====================================================================
# 関数名：db_final
# 機  能：データベースから切断する
#====================================================================
sub db_final {
	if($items_sth){
		$items_sth->finish();
	}
	$context->disconnect;
}


#====================================================================
# 関数名：db_count
# 機  能：該当件数を返す
#====================================================================
sub db_count {
	my $argments = @_[0];
	my $msg = "";

	my $select_sql = "SELECT code,mtcom,crtid,rnwid,mtstat,mtpermit,adate,udate,utime FROM";
	my $count_sql = "SELECT count(*) FROM";

	# Type-NII取得
	my $sql = "select code,tnm,enm from tn where key='TNII' order by code";
	my $q = qq($sql);

	my $tnii_sth = $context->prepare($q);
	eval {
		$tnii_sth->execute or die;
	};
	if($@){
		my $estr = $DBI::errstr;
		chomp($estr);
		my $msg = "データベースでエラーが発生しました。[".&common::convert($estr)."]";
		&common::log_write($common::LOG_ERROR,$msg);
		return(-1);
	}

	my $tncode;

	while (my $h = $tnii_sth->fetchrow_hashref("NAME_uc")) {
    	my $tncode = $h->{CODE} . "";
    	my $tnname = $h->{TNM};
    	my $tnname_e = $h->{ENM};
		$tnname = Encode::encode_utf8($tnname);
    	$tnii{$tncode} = "$tnname_e";
    	$tnii_name{$tnname} = "$tncode";
    	$tnii_name_e{$tnname_e} = "$tncode";
	}

	# ローカルType取得
	$sql = "select code,tnm,enm from tn where key='TORG' order by code";
	$q = qq($sql);

	my $torg_sth = $context->prepare($q);
	eval {
		$torg_sth->execute or die;
	};
	if($@){
		my $estr = $DBI::errstr;
		chomp($estr);
		my $msg = "データベースでエラーが発生しました。[".&common::convert($estr)."]";
		&common::log_write($common::LOG_ERROR,$msg);
		return(-1);
	}

	my $tncode;

	while (my $h = $torg_sth->fetchrow_hashref("NAME_uc")) {
    	my $tncode = $h->{CODE} . "";
    	my $tnname = $h->{TNM};
		$tnname = Encode::encode_utf8($tnname);
    	$torg{$tncode} = "$tnname";
    	$torg_name{$tnname} = "$tncode";
	}

	my $comm_coll_sql = "";
	$g_col_com_id = "";

	# 特定のNIIタイプのみ出力するを指定の場合
	my $comm_handle = $$argments{'nii-type'};
	if($comm_handle ne ""){
		# 名称からコード値を取得
		$g_col_com_id = $tnii_name{$comm_handle};
		if($g_col_com_id eq ""){
			# 日本語名称未設定の場合は英語名称から
			$g_col_com_id = $tnii_name_e{$comm_handle};
		}
	}

	# SQLの組み立て
	my $where_flg = 0;
	my $all_sql = "";
	my $sql = " (" .
        "  SELECT" .
        "    code,mtcom,crtid,rnwid,mtstat,mtpermit,adate,udate,utime, rownum as rn FROM mt" ;

	if($g_col_com_id ne ""){
		$sql .= " WHERE code in (SELECT code FROM md WHERE tag = 'Type' AND sstag = 'NII' AND data = '$g_col_com_id')";
		$where_flg = 1;
	}

	# 出力対象のステータス確認
	if($$argments{'metadata-status'} == 1){
		# 削除済みのみ出力
		if($where_flg == 0){
			$sql .= " WHERE ";
			$where_flg = 1;
		}
		else{
			$sql .= " AND ";
		}
		$sql .= "mtstat = 'D' ";
	}
	elsif($$argments{'metadata-status'} == 2){
		# 保留中(編集中)データのみ
		if($where_flg == 0){
			$sql .= " WHERE ";
			$where_flg = 1;
		}
		else{
			$sql .= " AND ";
		}
		$sql .= "mtstat = 'S' ";
	}
	elsif($$argments{'metadata-status'} == 3){
		# 保留中(編集中)データのみ
		if($where_flg == 0){
			$sql .= " WHERE ";
			$where_flg = 1;
		}
		else{
			$sql .= " AND ";
		}
		$sql .= "mtstat = 'H' ";
	}
	elsif($$argments{'metadata-status'} == 4){
		# 全て出力対象とする
	}
	else{
		# 承認/公開データのみ
		if($where_flg == 0){
			$sql .= " WHERE ";
			$where_flg = 1;
		}
		else{
			$sql .= " AND ";
		}
		$sql .= "mtstat = 'R' ";
	}

	# 差分更新抽出あり?
	my $difference_sql = "";
	if($$argments{'difference-mode'} == 1){ # 新規登録データのみ
        my $difference_date = $$argments{'difference-date'};
		$difference_sql = " AND adate >= '".$difference_date."' ";
	}
	elsif($$argments{'difference-mode'} == 2){  # 更新データのみ
		my $difference_date = $$argments{'difference-date'};
		$difference_sql = " AND udate >= '".$difference_date."' AND adate < '".$difference_date."' ";
	}
	$sql .= $difference_sql;

	$sql .= " ORDER BY code)";

	# データ出力位置が指定されている場合
	my $offset = $$argments{'start-position'} + 0;
	my $where1 = "";
	if($offset > 0){
		$where1 = " rn >= $offset";
	}
	else{
		$offset = 1;
	}

	# データ出力件数が指定されている場合
	my $where2 = "";
	if($$argments{'output-count'} > 0){
		my $limit = $$argments{'output-count'} + $offset - 1;
		if($limit > 0){
			$where2 = " rn <= $limit";
		}
	}

	if(($where1 ne "") || ($where2 ne "")){
		$sql .= " WHERE";
	}

	if($where1 ne ""){
		$sql .= $where1;
	}
	if($where2 ne ""){
		if($where1 ne ""){
			$sql .= " AND ";
		}
		$sql .= $where2;
	}

	# 件数取得用SQL
	$all_sql = "$count_sql$sql";

	$items_sth = $context->prepare($all_sql);
	eval {
		$items_sth->execute or die;
	};
	if($@){
		my $estr = $DBI::errstr;
		chomp($estr);
		my $msg = "データベースでエラーが発生しました。[".&common::convert($estr)."]";
		&common::log_write($common::LOG_ERROR,$msg);
		return(-1);
	}

	my $num=0;
	$items_sth->bind_col( 1, \$num);

	$items_sth->fetchrow_arrayref;
	$items_sth->finish;

	if($num <= 0){
		return($num);
	}

	# 実際にデータをセレクト
	$all_sql = "$select_sql$sql";

	$items_sth = $context->prepare($all_sql);
	eval {
		$items_sth->execute or die;
	};
	if($@){
		my $estr = $DBI::errstr;
		chomp($estr);
		my $msg = "データベースでエラーが発生しました。[".&common::convert($estr)."]";
		&common::log_write($common::LOG_ERROR,$msg);
		return(-1);
	}

	$items_sth->bind_col( 1, \$sth_item_id);
	$items_sth->bind_col( 2, \$sth_mtcom);
	$items_sth->bind_col( 3, \$sth_crtid);
	$items_sth->bind_col( 4, \$sth_rnwid);
	$items_sth->bind_col( 5, \$sth_mtstat);
	$items_sth->bind_col( 6, \$sth_mtpermit);
	$items_sth->bind_col( 7, \$sth_adate);
	$items_sth->bind_col( 8, \$sth_udate);
	$items_sth->bind_col( 9, \$sth_utime);

    # 日付のフォーマット変換の指定あり？
    @date_field = split(/,/,$$argments{'date-field'});
    for(my $i=0; $i<scalar(@date_field)-1; $i++){
        $date_field[$i] = &common::trim($date_field[$i]);
    }

	return($num);
}


#====================================================================
# 関数名：db_select
# 機  能：指定された SQL 文を実行し、結果を TSV の配列で返却する
#====================================================================
sub db_select {
    my ($sql, @var) = @_;
    my (@iterator) = ();
    my (@buffer) = ();
    my ($stmt, $fields, $lp);

    $stmt = $context->prepare($sql);

    if ( $#var != -1 ) {
        $lp = 1;
        foreach (@var) {
            $stmt->bind_param( $lp, $_ );
            $lp++;
        }
    }

	eval {
    	$stmt->execute or die;
	};
	if($@){
		my $estr = $DBI::errstr;
		chomp($estr);
		my $msg = "データベースでエラーが発生しました。[".&common::convert($estr)."]";
		&common::log_write($common::LOG_ERROR,$msg);
		return(-1);
	}

    while ( @buffer = $stmt->fetchrow_array ) {
        $fields = "";
        $lp = 0;
        foreach (@buffer) {
            $fields .= "\t" unless ($lp == 0);
			$fields = Encode::encode_utf8($_);
            $lp++;
        }
        push(@iterator, $fields);
    }
    $stmt->finish ;
    return @iterator;
}

#====================================================================
# 関数名：sanitaryColumns
# 機  能：UTF-8エンコーディングを行う
#====================================================================
sub sanitaryColumns {
    my (@colmuns) = @_ ;
    my (@results) = ();
    my ($buff) = "";

    for (@colmuns) {
   #     utf8::encode($_) if ( utf8::is_utf8($_) );
		Encode::encode_utf8($_) if ( Encode::is_utf8($_) );
    }
    return wantarray ? @colmuns : $colmuns[0];

}

#====================================================================
# 関数名：db_getTags
# 機  能：タグデータをTAB区切りで全出力
#====================================================================
sub db_getTags {

	my ($metadata_field_id,$element,$qualifier,$short_id);
	my $line = "";
	my $tags = "";

	my $tag_sql = "select distinct tagname from (select tag||'.'||stag||'.'||sstag as tagname from md) order by tagname";
	my @results = &db_select($tag_sql);

	my $i = 0;
	foreach my $result (@results) {
		if($line){
			$line .= "\t";
		}
		$line .= $result;

		$tag_table[$i] = $result;
		$i++;
	}

    # 日付のフォーマット変換の指定あり？
    my $df = "";
    for(my $i=0; $i<scalar(@date_field); $i++){
        if($date_field[$i]){
            $df .= "\t".&common::trim($date_field[$i]).".normalize";
        }
    }
    if($df){
        $line .= $df;
    }

	# WEKO用タグ：公開日とPOS_INDEXとITEM_KEYとファイルを付加
	$line .= "\tComment..\t作成者\t更新者\t作成日\t更新日\tmtstat\t公開日\tPOS_INDEX\tITEM_KEY\tファイル\tファイル（説明）\tWEKOアイテムタイプ";

	return $line;
}

#====================================================================
# 関数名：db_getData
# 機  能：1メタデータをTAB区切りで全出力
#====================================================================
sub db_getData {

	my $argments = @_[0];
	my $type_nii_list = @_[1];
	my $item_sth;

	$items_sth->fetchrow_arrayref;

	# 文字化け対策
	$sth_item_id = Encode::encode_utf8($sth_item_id);
	$sth_mtcom = Encode::encode_utf8($sth_mtcom);
	$sth_crtid = Encode::encode_utf8($sth_crtid);
	$sth_rnwid = Encode::encode_utf8($sth_rnwid);
	$sth_mtstat = Encode::encode_utf8($sth_mtstat);
	$sth_mtpermit = Encode::encode_utf8($sth_mtpermit);
	$sth_adate = Encode::encode_utf8($sth_adate);
	$sth_udate = Encode::encode_utf8($sth_udate);
	$sth_utime = Encode::encode_utf8($sth_utime);

	my $code="";
	my $tag="";
	my $stag="";
	my $sstag="";
	my $txtv="";
	my $wkidx=0;
	my $dtcnt=0;
	my @wkdata=();
	my $line="";
	my $pre_metaf_id="";
	my $msg = "";
	my $com_col_flg=0;
	my @comm_array = ();
	my $date_available = "";
	my $type_nii = "";
	my $pos_index = "";
	my %tags_array = ();
	my $date_available = "";
	my $weko_item = "";


	# データ取得
	my $sql = "SELECT code,tag,stag,sstag,data FROM md WHERE code = '$sth_item_id' ORDER BY tag,stag,sstag,seqno,dispseq";

	$item_sth = $context->prepare($sql);
	eval {
		$item_sth->execute or die;
	};
	if($@){
		my $estr = $DBI::errstr;
		chomp($estr);
		my $msg = "データベースでエラーが発生しました。[".&common::convert($estr)."]";
		&common::log_write($common::LOG_ERROR,$msg);
		exit;
	}
	$item_sth->bind_col(1, \$code);
	$item_sth->bind_col(2, \$tag);
	$item_sth->bind_col(3, \$stag);
	$item_sth->bind_col(4, \$sstag);
	$item_sth->bind_col(5, \$txtv);

	my $tag_name = "";
	my @date_field_value = ();
	$date_available = "";

	# 繰り返し項目はデリミタ(パラメタファイルで指定)で接続する
   	while ( $item_sth->fetchrow_arrayref ) {

		$code = Encode::encode_utf8($code);
		$tag = Encode::encode_utf8($tag);
		$stag = Encode::encode_utf8($stag);
		$sstag = Encode::encode_utf8($sstag);
		$txtv = Encode::encode_utf8($txtv);

		$tag_name = "$tag.$stag.$sstag";

		# Type.NIIをコードから名称へ変換
		if(($tag eq "Type") && ($sstag eq "NII")){
			$txtv = $tnii{"$txtv"};	# 出力は英語名称
			$type_nii = &common::normalization($txtv);

			# NII資源タイプからWEKOアイテムタイプ出力
			$weko_item = $$type_nii_list{$txtv};
		}
		# 独自Typeをコードから名称へ変換
		elsif(($tag eq "Type") && ($sstag eq "Local")){
			$txtv = $torg{$txtv};
		}
		# 独自Type(千葉大のみ)をコードから名称へ変換
		elsif(($tag eq "Type") && ($sstag eq "Chiba")){
			$txtv = $torg{$txtv};
		}
		# 独自Type(広島大のみ)をコードから名称へ変換
		elsif(($tag eq "Type") && ($sstag eq "HU")){
			$txtv = $torg{$txtv};
		}

		# 日付のフォーマット変換あり
		my $tname = $tag_name;
		$tname =~ s/\.$//;	# 末尾のピリオド外し
        for(my $i=0; $i<scalar(@date_field); $i++){
            # 指定のタグ名と同一か?
            if($date_field[$i] eq $tname){
                if($txtv){
                    my $buf = "";
					# 日付正規化関数呼び出し
					$buf = &common::date_normalize($txtv);
                    if($buf){
                        if($date_field_value[$i]){
                            $date_field_value[$i] .= $$argments{'delimiter'};
                        }
                        $date_field_value[$i] .= $buf;
                    }
                }
                last;
            }
        }


		# データの正規化
		#  - タブを半角スペースに変換
		#  - 改行コードを\nに変換
		#  - 制御コードを削除
		#  - データ中の「\n」->「\\n」
		$txtv = &common::normalization($txtv);
       	if ($pre_metaf_id eq $tag_name) {
           	$wkdata[$wkidx] .= $$argments{'delimiter'}."$txtv";
       	} else {
           	for (my $x = 0; $x <= scalar(@tag_table)-1; $x++) {
               	if ($tag_table[$x] eq $tag_name) {
                   	$wkdata[$x] = $txtv;
                   	$pre_metaf_id = $tag_name;
                   	$wkidx = $x;
                   	$dtcnt++;
                   	last;
               	}
           	}
       	}
		# 各タグのデータを配列にセット
		$tags_array{$tag_name} = $txtv;
   	}

	$item_sth->finish();

	if ($dtcnt > 0) {
		for (my $x = 0; $x <= scalar(@tag_table)-1; $x++) {
            $line .= $wkdata[$x];
			$line .= "\t";
		}

        # 日付のフォーマット変換あり
        for(my $i=0; $i<scalar(@date_field); $i++){
            $line .= $date_field_value[$i];
            $line .= "\t";
        }

		# コメント
		if($sth_mtcom){
			$sth_mtcom = &common::normalization($sth_mtcom);
		}
		$line .= $sth_mtcom;
		$line .= "\t";

		# 作成者
		$line .= $sth_crtid;
		$line .= "\t";

		# 更新者
		$line .= $sth_rnwid;
		$line .= "\t";

		# 作成日
		$date_available = "";
		if($sth_adate){
			my($year,$mon,$day) =
        		($sth_adate =~ /(\d{4})(\d{2})(\d{2})/);
			$date_available = $year . "-" . $mon . "-" . $day;
			# 日付正規化関数呼び出し
			$date_available = &common::date_normalize($sth_adate);
		}
		$line .= $date_available;
		$line .= "\t";

		# 更新日
		$date_available = "";
		if($sth_udate){
			my($year,$mon,$day) =
        		($sth_udate =~ /(\d{4})(\d{2})(\d{2})/);
			$date_available = $year . "-" . $mon . "-" . $day;
		}
		$line .= $date_available;
		$line .= "\t";

		# mtstat(状態)
		$line .= $sth_mtstat;
		$line .= "\t";

		# 公開日
		# 作成日を「公開日」とする
		$date_available = "";
		if($sth_adate){
			my($year,$mon,$day) =
        		($sth_adate =~ /(\d{4})(\d{2})(\d{2})/);
			$date_available = $year . "-" . $mon . "-" . $day;
		}

		$line .= $date_available;

		# インデックスツリーの出力
		# パラメタファイルで指定されたNIIタイプ別の階層で出力する
		# 連結文字は「/」
		@comm_array = ();
		$pos_index = "";
		my $pos_array = $$argments{$type_nii};
		foreach my $tag_val (@$pos_array) {
			my $tag_name = "";
			my $tag_edit = "";
			my $metaval = "";
			my $normalize_community = "";

			# タグ名と編集指定を分割
			($tag_name,$tag_edit) = split(/\|/,$tag_val,2);
			# タグ名の前後空白を削除
			$tag_name = &common::trim($tag_name);
			# 編集指定の[]を外す
			$tag_edit =~ s/^\[(.*?)\]$/$1/;


			$metaval = $tags_array{$tag_name};
#			$metaval = Encode::encode_utf8($metaval);

			# 編集指定がある場合は指定文字以降を削除
			if($tag_edit){
				my @edit_str = split(/\Q$tag_edit\E/,$metaval);
				$metaval = $edit_str[0];
			}

			# データの前後空白を削除
			$metaval = &common::trim($metaval);

			# Windows禁止文字のチェック
			$normalize_community = &common::prohibited_change($metaval);
			if($normalize_community ne $metaval){
				# 同じインデックスツリー名が続く場合が多いので一度出力したものは出力しないようにする
				if($prohibited_change_flg{$normalize_community} != 1){
					my $msg = "インデックスツリー名にWindowsで使用できない文字が含まれていたので変換しました。[".&common::convert($metaval)."] -> [".&common::convert($normalize_community)."]";
					&common::log_write($common::LOG_WARNING,$msg);
				}
				$prohibited_change_flg{$normalize_community} = 1;
			}
			if($normalize_community){
				if($pos_index){
					$pos_index .= "/";
				}
				$pos_index .= "$normalize_community";
			}
		}
		# 規定値(タイプNII)
		if($pos_index eq ""){
			$pos_index = $tags_array{'Type..NII'};
		}
		push(@comm_array,$pos_index);
	}

	return $sth_item_id,$line,$weko_item,\@comm_array;

}


#################################################################
# 本文ファイル取得、ディレクトリ作成、コピー
#################################################################
sub db_getFulltext {

    my ($item_key, $topdir, $argments) = @_;

	my $item_sth;
	my $fulltext_path = "";
	my $fulltext_file_path = "";
	my $fulltext_file = "";
	my @file_names = ();
	my $msg = "";
	my $normalize_fname = "";
	my @fname_array = ();
	my @fdescription_array = ();
	my $name = "";
	my $fname = "";

	$fulltext_path = $$argments{'fulltext-dir'};
	if(!$fulltext_path){
		my $msg = "本文データ格納先ディレクトリ名が設定されていません。";
		&common::log_write($common::LOG_ERROR,$msg);
		return(-1);
	}
	if(! -d $fulltext_path){
		my $msg =  "本文データ格納先ディレクトリ[$fulltext_path]は存在しません。";
		&common::log_write($common::LOG_ERROR,$msg);
		return(-1);
	} 

   	# 1次ファイル取得
   	my $sql = "SELECT count(*) FROM md WHERE code = '$item_key' AND tag = 'Identifier' AND sstag = 'FulltextURL'";

   	$item_sth = $context->prepare($sql);
	eval {
   		$item_sth->execute or die;
	};
	if($@){
		my $estr = $DBI::errstr;
		chomp($estr);
		my $msg = "データベースでエラーが発生しました。[".&common::convert($estr)."]";
		&common::log_write($common::LOG_ERROR,$msg);
		exit;
	}
	my $num = 0;
   	$item_sth->bind_col(1, \$num);

	$item_sth->fetchrow_arrayref;
	$item_sth->finish;

	if($num > 0){

   		# 1次ファイル取得
   		my $sql = "SELECT data FROM md WHERE code = '$item_key' AND tag = 'Identifier' AND sstag = 'FulltextURL'";

   		my $data = "";

   		$item_sth = $context->prepare($sql);
		eval {
   			$item_sth->execute or die;
		};
		if($@){
			my $estr = $DBI::errstr;
			chomp($estr);
			my $msg = "データベースでエラーが発生しました。[".&common::convert($estr)."]";
			&common::log_write($common::LOG_ERROR,$msg);
			exit;
		}
   		$item_sth->bind_col(1, \$data);

		# 「メタデータのみを出力する」の場合は実施しない
		if($$argments{'only-metadata'} != 1){
			# 本文ファイル格納用ディレクトリの作成
			$fulltext_file_path = "$topdir/$item_key";
			$! = "";
			mkdir $fulltext_file_path;
			if ($!) {
				# ディレクトリ作成エラー
				my $msg = "[".&common::convert($topdir)."]配下にディレクトリが作成できません。権限等を見直し再度実行してください。エラーコード[".&common::convert($!)."] ディレクトリ名[".&common::convert($topdir)."/".&common::convert($item_key)."]";
				&common::log_write($common::LOG_ERROR,$msg);
				return(-1);
			}
		}
	

   		# 本文ファイルをコピーする
   		while ( $item_sth->fetchrow_arrayref ) {

			$name = "";

			if ($data=~ /metadb\/up\/(.*)/) {
				$name = $1;
			}

			my @fnames = split(/\//,$name);
			$fname = $fnames[scalar(@fnames) - 1];

			# 複数ある場合は | で接続する
			if ($fname) {
				# Windows禁止文字のチェック
				$normalize_fname = &common::prohibited_change($fname);
				if($normalize_fname ne $fname){
					my $msg = "本文ファイル名にWindowsで使用できない文字が含まれていたので変換しました。ITEM_KEY:".&common::convert($item_key)." [".&common::convert($fname)."] -> [".&common::convert($normalize_fname)."]";
					&common::log_write($common::LOG_WARNING,$msg);
				}
				push(@fname_array,$normalize_fname);
				push(@fdescription_array,"");
			}

			# 「メタデータのみを出力する」の場合は実施しない
			if($$argments{'only-metadata'} != 1){
				if($name){
					# 本文ファイルのフルパス作成
					$fulltext_file = $fulltext_path . "/" . $name;

					if(-f $fulltext_file){
						# ファイルコピー
						$? = 0;
						$normalize_fname =~ s/["]/\\"/g;
						my $ret = system("cp \"$fulltext_file\" \"$fulltext_file_path/$normalize_fname\"");
						if ($? != 0){
							my $msg = "本文ファイルのコピーに失敗しました。エラーコード:".&common::convert($?)." ITEM_KEY:".&common::convert($item_key)." From:".&common::convert($fulltext_file)." To:".&common::convert($fulltext_file_path)."/".&common::convert($normalize_fname);
							&common::log_write($common::LOG_ERROR,$msg);
							return(-1);
						}
						my $ret = system("chmod 664 \"$fulltext_file_path/$normalize_fname\"");
					}
					else{
						# 本文データが存在しない -> エラーを出して続行
						my $msg = "本文ファイルが見つかりませんでした。ITEM_KEY:".&common::convert($item_key)." ファイル名:".&common::convert($name);
						&common::log_write($common::LOG_ERROR,$msg);
					}
				}
				else{
					# html等ファイルでないような場合
					# 本文データが存在しない -> エラーを出して続行
					my $msg = "本文ファイルが見つかりませんでした。ITEM_KEY:".&common::convert($item_key)." ファイル名:".&common::convert($data);
					&common::log_write($common::LOG_ERROR,$msg);
				}
			}
		}
	}

	$item_sth->finish();

	return(\@fname_array, \@fdescription_array);

}

#====================================================================
# 関数名：db_getEperson
# 機  能：epersonのデータをTAB区切りで全出力
#====================================================================
sub db_getEperson {

	my $topdir = @_[0];

	return(0);

}


1;

